#increase memory limit of R to 3000 MB
memory.limit(size = 4000)

#read in packages
library(rgdal)
library(sp)
library(SDMTools)
library(igraph)

#read in contemporary wetlands and pits shapefile
ContWetsAndPits <- readOGR("G:/NewBackup/GIS/RWBWetlandConnectivity/base/ContWetsAndPitsNonReservesGeographicPnts.shp", "ContWetsAndPitsNonReservesGeographicPnts", verbose = TRUE, p4s = NULL)
head(ContWetsAndPits@data)
ContWetsAndPits@data$NodeID <- 1:nrow(ContWetsAndPits@data)
retain <- c("FP_ID", "XCoord", "YCoord", "NodeID")
ContWetsAndPits@data <- ContWetsAndPits@data[retain]
head(ContWetsAndPits@data)

#read in and modify csv files of node connections forming dispersal paths at 4 dispersal distances
#####
#####
#####
#0.5 km
retain <- c("FromNode", "ToNode")
HalfKMPaths1 <- read.csv("HalfKMPathsNR1.csv")
HalfKMPaths1$FromNode <- HalfKMPaths1$unlist.FromNode1.
HalfKMPaths1$ToNode <- HalfKMPaths1$unlist.ToNode1.
HalfKMPaths1 <- HalfKMPaths1[retain]

HalfKMPaths2 <- read.csv("HalfKMPathsNR2.csv")
HalfKMPaths2$FromNode <- HalfKMPaths2$unlist.FromNode2.
HalfKMPaths2$ToNode <- HalfKMPaths2$unlist.ToNode2.
HalfKMPaths2 <- HalfKMPaths2[retain]

HalfKMPaths3 <- read.csv("HalfKMPathsNR3.csv")
HalfKMPaths3$FromNode <- HalfKMPaths3$unlist.FromNode3.
HalfKMPaths3$ToNode <- HalfKMPaths3$unlist.ToNode3.
HalfKMPaths3 <- HalfKMPaths3[retain]

HalfKMPaths4 <- read.csv("HalfKMPathsNR4.csv")
HalfKMPaths4$FromNode <- HalfKMPaths4$unlist.FromNode4.
HalfKMPaths4$ToNode <- HalfKMPaths4$unlist.ToNode4.
HalfKMPaths4 <- HalfKMPaths4[retain]

HalfKMPaths5 <- read.csv("HalfKMPathsNR5.csv")
HalfKMPaths5$FromNode <- HalfKMPaths5$unlist.FromNode5.
HalfKMPaths5$ToNode <- HalfKMPaths5$unlist.ToNode5.
HalfKMPaths5 <- HalfKMPaths5[retain]

HalfKMPaths6 <- read.csv("HalfKMPathsNR6.csv")
HalfKMPaths6$FromNode <- HalfKMPaths6$unlist.FromNode6.
HalfKMPaths6$ToNode <- HalfKMPaths6$unlist.ToNode6.
HalfKMPaths6 <- HalfKMPaths6[retain]

#combine 0.5 km connections into 1 data frame
HalfKMPathsCombined <- as.matrix(rbind(HalfKMPaths1, HalfKMPaths2, HalfKMPaths3, HalfKMPaths4, HalfKMPaths5, HalfKMPaths6))
HalfKMPathsCombined

#create network with nodes and associated edges
HalfKMNetwork <- graph(n = 9910, edges = HalfKMPathsCombined, directed = FALSE)

#calculate node and edge betweeness for graph (number of shortest paths going through node or edge)
HalfKMNodeBetweenness <- betweenness(HalfKMNetwork, v = V(HalfKMNetwork), directed = FALSE, weights = NULL, normalized = FALSE)
HalfKMNodeBetweenness
MeanNodeBetweennessHalfKM <- mean(HalfKMNodeBetweenness)
MeanNodeBetweennessHalfKM

HalfKMEdgeBetweenness <- edge.betweenness(HalfKMNetwork, e = E(HalfKMNetwork), directed = FALSE, weights = NULL)
HalfKMEdgeBetweenness
MeanEdgeBetweennessHalfKM <- mean(HalfKMEdgeBetweenness)
MeanEdgeBetweennessHalfKM

#calculate degree (number of adjacent edges for each node)
DegreeHalfKMNetwork <- degree(HalfKMNetwork, v = V(HalfKMNetwork), loops = TRUE)
DegreeHalfKMNetwork
MeanDegreeHalfKMNetwork <- mean(DegreeHalfKMNetwork)
MeanDegreeHalfKMNetwork

#find number of clusters (groups of connected graph components)
HalfKMNetworkNumberClusters <- no.clusters(HalfKMNetwork)
HalfKMNetworkNumberClusters

#find mean number of vertices per cluster
HalfKMNetworkClusters <- clusters(HalfKMNetwork)
summary(HalfKMNetworkClusters$csize)
HalfKMNetworkMeanClusterSize <- mean(HalfKMNetworkClusters$csize)
HalfKMNetworkMeanClusterSize

#calculate modularity
HalfKMNetworkModularity <- modularity(HalfKMNetwork, V(HalfKMNetwork))
HalfKMNetworkModularity

#find subcomponent (all other vertices reachable from specified vertice)
HalfKMNetworkSC <- list(NA)
NumberReachedHalfKM <- list(NA)
for(i in 1:nrow(ContWetsAndPits@data)){
	HalfKMNetworkSC[[i]] <- subcomponent(HalfKMNetwork, v = i)
	NumberReachedHalfKM[[i]] <- length(HalfKMNetworkSC[[i]]) - 1
}
NumberReachedHalfKM <- unlist(NumberReachedHalfKM)
NumberReachedHalfKMMean <- mean(NumberReachedHalfKM)
NumberReachedHalfKMMean

#edge betweeness community (creates hierarchical graph by removing edges most traveled when reaching other vertices via algorithm)
HalfKMNetworkEBC <- edge.betweenness.community(HalfKMNetwork, edge.betweenness = TRUE, merges = TRUE, bridges = TRUE)

#calculate eigenvector centrality scores of network positions (high scores = vertices connected to other highly connected which are connected to other highly connected etc...)
HalfKMNetworkEVCENT <- evcent(HalfKMNetwork, directed = FALSE, scale = FALSE)
HalfKMNetworkEVCENT

#calculate graph density (ratio of number of edges to number of possible edges)
HalfKMNetworkGraphDensity <- graph.density(HalfKMNetwork, loops = TRUE)
HalfKMNetworkGraphDensity

#identify neighbors of each vertice
HalfKMNetworkNeighbors <- list(NA)
for (i in 1:nrow(ContWetsAndPits@data)){
	HalfKMNetworkNeighbors[[i]] <- neighbors(HalfKMNetwork, v = i)
}
HalfKMNetworkNeighbors

#calculate neighborhood size (number of interconnected vertices in each neighborhood)
HalfKMNetworkNeighborSize <- neighborhood.size(HalfKMNetwork, order = 1, nodes = V(HalfKMNetwork))
HalfKMNetworkNeighborSize

#calculate graph cohesion (vertex connectivity of graph)
HalfKMNetworkGraphCohesion <- graph.cohesion(HalfKMNetwork)
HalfKMNetworkGraphCohesion

#calculate average shortest path length between connected vertices
HalfKMNetworkAPL <- average.path.length(HalfKMNetwork, directed = FALSE, unconnected = TRUE)
HalfKMNetworkAPL

#generate histogram of shortest path lengths
HalfKMNetworkAPLHist <- path.length.hist(HalfKMNetwork, directed = FALSE)
HalfKMNetworkAPLHist

#calculate degree and betweenness centrality for graph, without removing any nodes
HalfKMNetworkDegreeCentrality <- centralization.degree(HalfKMNetwork, loops = TRUE, normalized = TRUE)
HalfKMNetworkDegreeCentrality <- HalfKMNetworkDegreeCentrality$centralization
HalfKMNetworkDegreeCentrality

#add a degree and betweenness centralization importance scores for each vertice
#####
#####
#####
#part 1
NodeRemove1 <- list(NA)
DegreeCentralizationRemoval1 <- list(NA)
DegCentImp1 <- list(NA)
for(i in 1:3300){
	NodeRemove1[[i]] <- HalfKMNetwork
	NodeRemove1[[i]] <- NodeRemove1[[i]] - i
	DegreeCentralizationRemoval1[[i]] <- centralization.degree(NodeRemove1[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval1[[i]] <- DegreeCentralizationRemoval1[[i]]$centralization
	DegCentImp1[[i]] <- ((HalfKMNetworkDegreeCentrality - DegreeCentralizationRemoval1[[i]])/HalfKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp1 <- unlist(DegCentImp1)
DegCentImp1 <- data.frame(DegCentImp1)
DegCentImp1$centralization <- DegCentImp1[,1]
DegCentImp1$NodeID <- ContWetsAndPits@data[1:3300,]$NodeID
DegCentImp1$FP_ID <- ContWetsAndPits@data[1:3300,]$FP_ID
DegCentImp1 <- DegCentImp1[,-1]
head(DegCentImp1)
write.csv(DegCentImp1, "DegCentImpHalfKM1.csv")

#####
#####
#####
#part 2
NodeRemove2 <- list(NA)
DegreeCentralizationRemoval2 <- list(NA)
DegCentImp2 <- list(NA)
for(i in 1:3300){
	NodeRemove2[[i]] <- HalfKMNetwork
	NodeRemove2[[i]] <- NodeRemove2[[i]] - (3300 + i)
	DegreeCentralizationRemoval2[[i]] <- centralization.degree(NodeRemove2[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval2[[i]] <- DegreeCentralizationRemoval2[[i]]$centralization
	DegCentImp2[[i]] <- ((HalfKMNetworkDegreeCentrality - DegreeCentralizationRemoval2[[i]])/HalfKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp2 <- unlist(DegCentImp2)
DegCentImp2 <- data.frame(DegCentImp2)
DegCentImp2$centralization <- DegCentImp2[,1]
DegCentImp2$NodeID <- ContWetsAndPits@data[3301:6600,]$NodeID
DegCentImp2$FP_ID <- ContWetsAndPits@data[3301:6600,]$FP_ID
DegCentImp2 <- DegCentImp2[,-1]
head(DegCentImp2)
write.csv(DegCentImp2, "DegCentImpHalfKM2.csv")

#####
#####
#####
#part 3
NodeRemove3 <- list(NA)
DegreeCentralizationRemoval3 <- list(NA)
DegCentImp3 <- list(NA)
for(i in 1:3310){
	NodeRemove3[[i]] <- HalfKMNetwork
	NodeRemove3[[i]] <- NodeRemove3[[i]] - (6600 + i)
	DegreeCentralizationRemoval3[[i]] <- centralization.degree(NodeRemove3[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval3[[i]] <- DegreeCentralizationRemoval3[[i]]$centralization
	DegCentImp3[[i]] <- ((HalfKMNetworkDegreeCentrality - DegreeCentralizationRemoval3[[i]])/HalfKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp3 <- unlist(DegCentImp3)
DegCentImp3 <- data.frame(DegCentImp3)
DegCentImp3$centralization <- DegCentImp3[,1]
DegCentImp3$NodeID <- ContWetsAndPits@data[6601:9910,]$NodeID
DegCentImp3$FP_ID <- ContWetsAndPits@data[6601:9910,]$FP_ID
DegCentImp3 <- DegCentImp3[,-1]
head(DegCentImp3)
write.csv(DegCentImp3, "DegCentImpHalfKM3.csv")

#read back in degree centrality importance scores
DegCentImp1 <- read.csv("DegCentImpHalfKM1.csv", header = TRUE)
DegCentImp1 <- DegCentImp1[,-1]
DegCentImp2 <- read.csv("DegCentImpHalfKM2.csv", header = TRUE)
DegCentImp2 <- DegCentImp2[,-1]
DegCentImp3 <- read.csv("DegCentImpHalfKM3.csv", header = TRUE)
DegCentImp3 <- DegCentImp3[,-1]

#combine csv files
DegCentImp <- rbind(DegCentImp1, DegCentImp2, DegCentImp3)

#add new outputs to attribute table
ContWetsAndPits@data$Degree <- DegreeHalfKMNetwork
ContWetsAndPits@data$DegCentImp <- DegCentImp$centralization
head(ContWetsAndPits@data)

#write updated spatial polygons data frame as an ESRI shapefile
writeOGR(ContWetsAndPits, "ContWetsAndPitsNonReservesHalfKM", "ContWetsAndPitsNonReservesHalfKM", driver = 'ESRI Shapefile', verbose = TRUE)


#####
#####
#####
#1.0 km
retain <- c("FromNode", "ToNode")
OneKMPaths1 <- read.csv("OneKMPathsNR1.csv")
OneKMPaths1$FromNode <- OneKMPaths1$unlist.FromNode1.
OneKMPaths1$ToNode <- OneKMPaths1$unlist.ToNode1.
OneKMPaths1 <- OneKMPaths1[retain]

OneKMPaths2 <- read.csv("OneKMPathsNR2.csv")
OneKMPaths2$FromNode <- OneKMPaths2$unlist.FromNode2.
OneKMPaths2$ToNode <- OneKMPaths2$unlist.ToNode2.
OneKMPaths2 <- OneKMPaths2[retain]

OneKMPaths3 <- read.csv("OneKMPathsNR3.csv")
OneKMPaths3$FromNode <- OneKMPaths3$unlist.FromNode3.
OneKMPaths3$ToNode <- OneKMPaths3$unlist.ToNode3.
OneKMPaths3 <- OneKMPaths3[retain]

OneKMPaths4 <- read.csv("OneKMPathsNR4.csv")
OneKMPaths4$FromNode <- OneKMPaths4$unlist.FromNode4.
OneKMPaths4$ToNode <- OneKMPaths4$unlist.ToNode4.
OneKMPaths4 <- OneKMPaths4[retain]

OneKMPaths5 <- read.csv("OneKMPathsNR5.csv")
OneKMPaths5$FromNode <- OneKMPaths5$unlist.FromNode5.
OneKMPaths5$ToNode <- OneKMPaths5$unlist.ToNode5.
OneKMPaths5 <- OneKMPaths5[retain]

OneKMPaths6 <- read.csv("OneKMPathsNR6.csv")
OneKMPaths6$FromNode <- OneKMPaths6$unlist.FromNode6.
OneKMPaths6$ToNode <- OneKMPaths6$unlist.ToNode6.
OneKMPaths6 <- OneKMPaths6[retain]

#combine 1.0 km connections into 1 data frame
OneKMPathsCombined <- as.matrix(rbind(OneKMPaths1, OneKMPaths2, OneKMPaths3, OneKMPaths4, OneKMPaths5, OneKMPaths6))
OneKMPathsCombined

#create network with nodes and associated edges
OneKMNetwork <- graph(n = 9910, edges = OneKMPathsCombined, directed = FALSE)

#calculate node and edge betweeness for graph (number of shortest paths going through node or edge)
OneKMNodeBetweenness <- betweenness(OneKMNetwork, v = V(OneKMNetwork), directed = FALSE, weights = NULL, normalized = FALSE)
OneKMNodeBetweenness
MeanNodeBetweennessOneKM <- mean(OneKMNodeBetweenness)
MeanNodeBetweennessOneKM

OneKMEdgeBetweenness <- edge.betweenness(OneKMNetwork, e = E(OneKMNetwork), directed = FALSE, weights = NULL)
OneKMEdgeBetweenness
MeanEdgeBetweennessOneKM <- mean(OneKMEdgeBetweenness)
MeanEdgeBetweennessOneKM

#calculate degree (number of adjacent edges for each node)
DegreeOneKMNetwork <- degree(OneKMNetwork, v = V(OneKMNetwork), loops = TRUE)
DegreeOneKMNetwork
MeanDegreeOneKMNetwork <- mean(DegreeOneKMNetwork)
MeanDegreeOneKMNetwork

#find number of clusters (groups of connected graph components)
OneKMNetworkNumberClusters <- no.clusters(OneKMNetwork)
OneKMNetworkNumberClusters

#find mean number of vertices per cluster
OneKMNetworkClusters <- clusters(OneKMNetwork)
summary(OneKMNetworkClusters$csize)
OneKMNetworkMeanClusterSize <- mean(OneKMNetworkClusters$csize)
OneKMNetworkMeanClusterSize

#calculate modularity
OneKMNetworkModularity <- modularity(OneKMNetwork, V(OneKMNetwork))
OneKMNetworkModularity

#find subcomponent (all other vertices reachable from specified vertice)
OneKMNetworkSC <- list(NA)
NumberReachedOneKM <- list(NA)
for(i in 1:nrow(ContWetsAndPits@data)){
	OneKMNetworkSC[[i]] <- subcomponent(OneKMNetwork, v = i)
	NumberReachedOneKM[[i]] <- length(OneKMNetworkSC[[i]]) - 1
}
NumberReachedOneKM <- unlist(NumberReachedOneKM)
NumberReachedOneKMMean <- mean(NumberReachedOneKM)
NumberReachedOneKMMean

#edge betweeness community (creates hierarchical graph by removing edges most traveled when reaching other vertices via algorithm)
OneKMNetworkEBC <- edge.betweenness.community(OneKMNetwork, edge.betweenness = TRUE, merges = TRUE, bridges = TRUE)

#calculate eigenvector centrality scores of network positions (high scores = vertices connected to other highly connected which are connected to other highly connected etc...)
OneKMNetworkEVCENT <- evcent(OneKMNetwork, directed = FALSE, scale = FALSE)
OneKMNetworkEVCENT

#calculate graph density (ratio of number of edges to number of possible edges)
OneKMNetworkGraphDensity <- graph.density(OneKMNetwork, loops = TRUE)
OneKMNetworkGraphDensity

#identify neighbors of each vertice
OneKMNetworkNeighbors <- list(NA)
for (i in 1:nrow(ContWetsAndPits@data)){
	OneKMNetworkNeighbors[[i]] <- neighbors(OneKMNetwork, v = i)
}
OneKMNetworkNeighbors

#calculate neighborhood size (number of interconnected vertices in each neighborhood)
OneKMNetworkNeighborSize <- neighborhood.size(OneKMNetwork, order = 1, nodes = V(OneKMNetwork))
OneKMNetworkNeighborSize

#calculate graph cohesion (vertex connectivity of graph)
OneKMNetworkGraphCohesion <- graph.cohesion(OneKMNetwork)
OneKMNetworkGraphCohesion

#calculate average shortest path length between connected vertices
OneKMNetworkAPL <- average.path.length(OneKMNetwork, directed = FALSE, unconnected = TRUE)
OneKMNetworkAPL

#generate histogram of shortest path lengths
OneKMNetworkAPLHist <- path.length.hist(OneKMNetwork, directed = FALSE)
OneKMNetworkAPLHist

#update and re-write attribute table, prior to dealing with memory allocation issues
ContWetsAndPits@data$Degree <- DegreeOneKMNetwork
ContWetsAndPits@data$NodeBetween <- OneKMNodeBetweenness
writeOGR(ContWetsAndPits, "ContWetsAndPitsOneKMNetwork1", "ContWetsAndPitsOneKMNetwork1", driver = 'ESRI Shapefile', verbose = TRUE)

#calculate degree and betweenness centrality for graph, without removing any nodes
OneKMNetworkDegreeCentrality <- centralization.degree(OneKMNetwork, loops = TRUE, normalized = TRUE)
OneKMNetworkDegreeCentrality <- OneKMNetworkDegreeCentrality$centralization
OneKMNetworkDegreeCentrality

#add a degree and betweenness centralization importance scores for each vertice
#####
#####
#####
#part 1
NodeRemove1 <- list(NA)
DegreeCentralizationRemoval1 <- list(NA)
DegCentImp1 <- list(NA)
for(i in 1:3300){
	NodeRemove1[[i]] <- OneKMNetwork
	NodeRemove1[[i]] <- NodeRemove1[[i]] - i
	DegreeCentralizationRemoval1[[i]] <- centralization.degree(NodeRemove1[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval1[[i]] <- DegreeCentralizationRemoval1[[i]]$centralization
	DegCentImp1[[i]] <- ((OneKMNetworkDegreeCentrality - DegreeCentralizationRemoval1[[i]])/OneKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp1 <- unlist(DegCentImp1)
DegCentImp1 <- data.frame(DegCentImp1)
DegCentImp1$centralization <- DegCentImp1[,1]
DegCentImp1$NodeID <- ContWetsAndPits@data[1:3300,]$NodeID
DegCentImp1$FP_ID <- ContWetsAndPits@data[1:3300,]$FP_ID
DegCentImp1 <- DegCentImp1[,-1]
head(DegCentImp1)
write.csv(DegCentImp1, "DegCentImpOneKM1.csv")

#####
#####
#####
#part 2
NodeRemove2 <- list(NA)
DegreeCentralizationRemoval2 <- list(NA)
DegCentImp2 <- list(NA)
for(i in 1:3300){
	NodeRemove2[[i]] <- OneKMNetwork
	NodeRemove2[[i]] <- NodeRemove2[[i]] - (3300 + i)
	DegreeCentralizationRemoval2[[i]] <- centralization.degree(NodeRemove2[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval2[[i]] <- DegreeCentralizationRemoval2[[i]]$centralization
	DegCentImp2[[i]] <- ((OneKMNetworkDegreeCentrality - DegreeCentralizationRemoval2[[i]])/OneKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp2 <- unlist(DegCentImp2)
DegCentImp2 <- data.frame(DegCentImp2)
DegCentImp2$centralization <- DegCentImp2[,1]
DegCentImp2$NodeID <- ContWetsAndPits@data[3301:6600,]$NodeID
DegCentImp2$FP_ID <- ContWetsAndPits@data[3301:6600,]$FP_ID
DegCentImp2 <- DegCentImp2[,-1]
head(DegCentImp2)
write.csv(DegCentImp2, "DegCentImpOneKM2.csv")

#####
#####
#####
#part 3
NodeRemove3 <- list(NA)
DegreeCentralizationRemoval3 <- list(NA)
DegCentImp3 <- list(NA)
for(i in 1:3310){
	NodeRemove3[[i]] <- OneKMNetwork
	NodeRemove3[[i]] <- NodeRemove3[[i]] - (6600 + i)
	DegreeCentralizationRemoval3[[i]] <- centralization.degree(NodeRemove3[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval3[[i]] <- DegreeCentralizationRemoval3[[i]]$centralization
	DegCentImp3[[i]] <- ((OneKMNetworkDegreeCentrality - DegreeCentralizationRemoval3[[i]])/OneKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp3 <- unlist(DegCentImp3)
DegCentImp3 <- data.frame(DegCentImp3)
DegCentImp3$centralization <- DegCentImp3[,1]
DegCentImp3$NodeID <- ContWetsAndPits@data[6601:9910,]$NodeID
DegCentImp3$FP_ID <- ContWetsAndPits@data[6601:9910,]$FP_ID
DegCentImp3 <- DegCentImp3[,-1]
head(DegCentImp3)
write.csv(DegCentImp3, "DegCentImpOneKM3.csv")

#read back in degree centrality importance scores
DegCentImp1 <- read.csv("DegCentImpOneKM1.csv", header = TRUE)
DegCentImp1 <- DegCentImp1[,-1]
DegCentImp2 <- read.csv("DegCentImpOneKM2.csv", header = TRUE)
DegCentImp2 <- DegCentImp2[,-1]
DegCentImp3 <- read.csv("DegCentImpOneKM3.csv", header = TRUE)
DegCentImp3 <- DegCentImp3[,-1]

#combine csv files
DegCentImp <- rbind(DegCentImp1, DegCentImp2, DegCentImp3)

#add new outputs to attribute table
ContWetsAndPits@data$Degree <- DegreeOneKMNetwork
ContWetsAndPits@data$DegCentImp <- DegCentImp$centralization
head(ContWetsAndPits@data)

#write updated spatial polygons data frame as an ESRI shapefile
writeOGR(ContWetsAndPits, "ContWetsAndPitsNonReservesOneKM", "ContWetsAndPitsNonReservesOneKM", driver = 'ESRI Shapefile', verbose = TRUE)

#####
#####
#####
#1.5 km
retain <- c("FromNode", "ToNode")
OneAndHalfKMPaths1 <- read.csv("OneAndHalfKMPathsNR1.csv")
OneAndHalfKMPaths1$FromNode <- OneAndHalfKMPaths1$unlist.FromNode1.
OneAndHalfKMPaths1$ToNode <- OneAndHalfKMPaths1$unlist.ToNode1.
OneAndHalfKMPaths1 <- OneAndHalfKMPaths1[retain]

OneAndHalfKMPaths2 <- read.csv("OneAndHalfKMPathsNR2.csv")
OneAndHalfKMPaths2$FromNode <- OneAndHalfKMPaths2$unlist.FromNode2.
OneAndHalfKMPaths2$ToNode <- OneAndHalfKMPaths2$unlist.ToNode2.
OneAndHalfKMPaths2 <- OneAndHalfKMPaths2[retain]

OneAndHalfKMPaths3 <- read.csv("OneAndHalfKMPathsNR3.csv")
OneAndHalfKMPaths3$FromNode <- OneAndHalfKMPaths3$unlist.FromNode3.
OneAndHalfKMPaths3$ToNode <- OneAndHalfKMPaths3$unlist.ToNode3.
OneAndHalfKMPaths3 <- OneAndHalfKMPaths3[retain]

OneAndHalfKMPaths4 <- read.csv("OneAndHalfKMPathsNR4.csv")
OneAndHalfKMPaths4$FromNode <- OneAndHalfKMPaths4$unlist.FromNode4.
OneAndHalfKMPaths4$ToNode <- OneAndHalfKMPaths4$unlist.ToNode4.
OneAndHalfKMPaths4 <- OneAndHalfKMPaths4[retain]

OneAndHalfKMPaths5 <- read.csv("OneAndHalfKMPathsNR5.csv")
OneAndHalfKMPaths5$FromNode <- OneAndHalfKMPaths5$unlist.FromNode5.
OneAndHalfKMPaths5$ToNode <- OneAndHalfKMPaths5$unlist.ToNode5.
OneAndHalfKMPaths5 <- OneAndHalfKMPaths5[retain]

OneAndHalfKMPaths6 <- read.csv("OneAndHalfKMPathsNR6.csv")
OneAndHalfKMPaths6$FromNode <- OneAndHalfKMPaths6$unlist.FromNode6.
OneAndHalfKMPaths6$ToNode <- OneAndHalfKMPaths6$unlist.ToNode6.
OneAndHalfKMPaths6 <- OneAndHalfKMPaths6[retain]

#combine 1.5 km connections into 1 data frame
OneAndHalfKMPathsCombined <- as.matrix(rbind(OneAndHalfKMPaths1, OneAndHalfKMPaths2, OneAndHalfKMPaths3, OneAndHalfKMPaths4, OneAndHalfKMPaths5, OneAndHalfKMPaths6))
OneAndHalfKMPathsCombined

#create network with nodes and associated edges
OneAndHalfKMNetwork <- graph(n = 9910, edges = OneAndHalfKMPathsCombined, directed = FALSE)

#calculate node and edge betweeness for graph (number of shortest paths going through node or edge)
OneAndHalfKMNodeBetweenness <- betweenness(OneAndHalfKMNetwork, v = V(OneAndHalfKMNetwork), directed = FALSE, weights = NULL, normalized = FALSE)
OneAndHalfKMNodeBetweenness
MeanNodeBetweennessOneAndHalfKM <- mean(OneAndHalfKMNodeBetweenness)
MeanNodeBetweennessOneAndHalfKM

OneAndHalfKMEdgeBetweenness <- edge.betweenness(OneAndHalfKMNetwork, e = E(OneAndHalfKMNetwork), directed = FALSE, weights = NULL)
OneAndHalfKMEdgeBetweenness
MeanEdgeBetweennessOneAndHalfKM <- mean(OneAndHalfKMEdgeBetweenness)
MeanEdgeBetweennessOneAndHalfKM

#calculate degree (number of adjacent edges for each node)
DegreeOneAndHalfKMNetwork <- degree(OneAndHalfKMNetwork, v = V(OneAndHalfKMNetwork), loops = TRUE)
DegreeOneAndHalfKMNetwork
MeanDegreeOneAndHalfKMNetwork <- mean(DegreeOneAndHalfKMNetwork)
MeanDegreeOneAndHalfKMNetwork

#find number of clusters (groups of connected graph components)
OneAndHalfKMNetworkNumberClusters <- no.clusters(OneAndHalfKMNetwork)
OneAndHalfKMNetworkNumberClusters

#find mean number of vertices per cluster
OneAndHalfKMNetworkClusters <- clusters(OneAndHalfKMNetwork)
summary(OneAndHalfKMNetworkClusters$csize)
OneAndHalfKMNetworkMeanClusterSize <- mean(OneAndHalfKMNetworkClusters$csize)
OneAndHalfKMNetworkMeanClusterSize

#calculate modularity
OneAndHalfKMNetworkModularity <- modularity(OneAndHalfKMNetwork, V(OneAndHalfKMNetwork))
OneAndHalfKMNetworkModularity

#find subcomponent (all other vertices reachable from specified vertice)
OneAndHalfKMNetworkSC <- list(NA)
NumberReachedOneAndHalfKM <- list(NA)
for(i in 1:nrow(ContWetsAndPits@data)){
	OneAndHalfKMNetworkSC[[i]] <- subcomponent(OneAndHalfKMNetwork, v = i)
	NumberReachedOneAndHalfKM[[i]] <- length(OneAndHalfKMNetworkSC[[i]]) - 1
}
NumberReachedOneAndHalfKM <- unlist(NumberReachedOneAndHalfKM)
NumberReachedOneAndHalfKMMean <- mean(NumberReachedOneAndHalfKM)
NumberReachedOneAndHalfKMMean

#edge betweeness community (creates hierarchical graph by removing edges most traveled when reaching other vertices via algorithm)
OneAndHalfKMNetworkEBC <- edge.betweenness.community(OneAndHalfKMNetwork, edge.betweenness = TRUE, merges = TRUE, bridges = TRUE)

#calculate eigenvector centrality scores of network positions (high scores = vertices connected to other highly connected which are connected to other highly connected etc...)
OneAndHalfKMNetworkEVCENT <- evcent(OneAndHalfKMNetwork, directed = FALSE, scale = FALSE)
OneAndHalfKMNetworkEVCENT

#calculate graph density (ratio of number of edges to number of possible edges)
OneAndHalfKMNetworkGraphDensity <- graph.density(OneAndHalfKMNetwork, loops = TRUE)
OneAndHalfKMNetworkGraphDensity

#identify neighbors of each vertice
OneAndHalfKMNetworkNeighbors <- list(NA)
for (i in 1:nrow(ContWetsAndPits@data)){
	OneAndHalfKMNetworkNeighbors[[i]] <- neighbors(OneAndHalfKMNetwork, v = i)
}
OneAndHalfKMNetworkNeighbors

#calculate neighborhood size (number of interconnected vertices in each neighborhood)
OneAndHalfKMNetworkNeighborSize <- neighborhood.size(OneAndHalfKMNetwork, order = 1, nodes = V(OneAndHalfKMNetwork))
OneAndHalfKMNetworkNeighborSize

#calculate graph cohesion (vertex connectivity of graph)
OneAndHalfKMNetworkGraphCohesion <- graph.cohesion(OneAndHalfKMNetwork)
OneAndHalfKMNetworkGraphCohesion

#calculate average shortest path length between connected vertices
OneAndHalfKMNetworkAPL <- average.path.length(OneAndHalfKMNetwork, directed = FALSE, unconnected = TRUE)
OneAndHalfKMNetworkAPL

#generate histogram of shortest path lengths
OneAndHalfKMNetworkAPLHist <- path.length.hist(OneAndHalfKMNetwork, directed = FALSE)
OneAndHalfKMNetworkAPLHist

#update and re-write attribute table, prior to dealing with memory allocation issues
ContWetsAndPits@data$Degree <- DegreeOneAndHalfKMNetwork
ContWetsAndPits@data$NodeBetween <- OneAndHalfKMNodeBetweenness
writeOGR(ContWetsAndPits, "ContWetsAndPitsOneAndHalfKMNetwork1", "ContWetsAndPitsOneAndHalfKMNetwork1", driver = 'ESRI Shapefile', verbose = TRUE)

#calculate degree and betweenness centrality for graph, without removing any nodes
OneAndHalfKMNetworkDegreeCentrality <- centralization.degree(OneAndHalfKMNetwork, loops = TRUE, normalized = TRUE)
OneAndHalfKMNetworkDegreeCentrality <- OneAndHalfKMNetworkDegreeCentrality$centralization
OneAndHalfKMNetworkDegreeCentrality

#add a degree and betweenness centralization importance scores for each vertice
#####
#####
#####
#part 1
NodeRemove1 <- list(NA)
DegreeCentralizationRemoval1 <- list(NA)
DegCentImp1 <- list(NA)
for(i in 1:1500){
	NodeRemove1[[i]] <- OneAndHalfKMNetwork
	NodeRemove1[[i]] <- NodeRemove1[[i]] - i
	DegreeCentralizationRemoval1[[i]] <- centralization.degree(NodeRemove1[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval1[[i]] <- DegreeCentralizationRemoval1[[i]]$centralization
	DegCentImp1[[i]] <- ((OneAndHalfKMNetworkDegreeCentrality - DegreeCentralizationRemoval1[[i]])/OneAndHalfKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp1 <- unlist(DegCentImp1)
DegCentImp1 <- data.frame(DegCentImp1)
DegCentImp1$centralization <- DegCentImp1[,1]
DegCentImp1$NodeID <- ContWetsAndPits@data[1:1500,]$NodeID
DegCentImp1$FP_ID <- ContWetsAndPits@data[1:1500,]$FP_ID
DegCentImp1 <- DegCentImp1[,-1]
head(DegCentImp1)
write.csv(DegCentImp1, "DegCentImpOneAndHalfKM1.csv")

#####
#####
#####
#part 2
NodeRemove2 <- list(NA)
DegreeCentralizationRemoval2 <- list(NA)
DegCentImp2 <- list(NA)
for(i in 1:1500){
	NodeRemove2[[i]] <- OneAndHalfKMNetwork
	NodeRemove2[[i]] <- NodeRemove2[[i]] - (1500 + i)
	DegreeCentralizationRemoval2[[i]] <- centralization.degree(NodeRemove2[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval2[[i]] <- DegreeCentralizationRemoval2[[i]]$centralization
	DegCentImp2[[i]] <- ((OneAndHalfKMNetworkDegreeCentrality - DegreeCentralizationRemoval2[[i]])/OneAndHalfKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp2 <- unlist(DegCentImp2)
DegCentImp2 <- data.frame(DegCentImp2)
DegCentImp2$centralization <- DegCentImp2[,1]
DegCentImp2$NodeID <- ContWetsAndPits@data[1501:3000,]$NodeID
DegCentImp2$FP_ID <- ContWetsAndPits@data[1501:3000,]$FP_ID
DegCentImp2 <- DegCentImp2[,-1]
head(DegCentImp2)
write.csv(DegCentImp2, "DegCentImpOneAndHalfKM2.csv")

#####
#####
#####
#part 3
NodeRemove3 <- list(NA)
DegreeCentralizationRemoval3 <- list(NA)
DegCentImp3 <- list(NA)
for(i in 1:1500){
	NodeRemove3[[i]] <- OneAndHalfKMNetwork
	NodeRemove3[[i]] <- NodeRemove3[[i]] - (3000 + i)
	DegreeCentralizationRemoval3[[i]] <- centralization.degree(NodeRemove3[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval3[[i]] <- DegreeCentralizationRemoval3[[i]]$centralization
	DegCentImp3[[i]] <- ((OneAndHalfKMNetworkDegreeCentrality - DegreeCentralizationRemoval3[[i]])/OneAndHalfKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp3 <- unlist(DegCentImp3)
DegCentImp3 <- data.frame(DegCentImp3)
DegCentImp3$centralization <- DegCentImp3[,1]
DegCentImp3$NodeID <- ContWetsAndPits@data[3001:4500,]$NodeID
DegCentImp3$FP_ID <- ContWetsAndPits@data[3001:4500,]$FP_ID
DegCentImp3 <- DegCentImp3[,-1]
head(DegCentImp3)
write.csv(DegCentImp3, "DegCentImpOneAndHalfKM3.csv")

#####
#####
#####
#part 4
NodeRemove4 <- list(NA)
DegreeCentralizationRemoval4 <- list(NA)
DegCentImp4 <- list(NA)
for(i in 1:1500){
	NodeRemove4[[i]] <- OneAndHalfKMNetwork
	NodeRemove4[[i]] <- NodeRemove4[[i]] - (4500 + i)
	DegreeCentralizationRemoval4[[i]] <- centralization.degree(NodeRemove4[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval4[[i]] <- DegreeCentralizationRemoval4[[i]]$centralization
	DegCentImp4[[i]] <- ((OneAndHalfKMNetworkDegreeCentrality - DegreeCentralizationRemoval4[[i]])/OneAndHalfKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp4 <- unlist(DegCentImp4)
DegCentImp4 <- data.frame(DegCentImp4)
DegCentImp4$centralization <- DegCentImp4[,1]
DegCentImp4$NodeID <- ContWetsAndPits@data[4501:6000,]$NodeID
DegCentImp4$FP_ID <- ContWetsAndPits@data[4501:6000,]$FP_ID
DegCentImp4 <- DegCentImp4[,-1]
head(DegCentImp4)
write.csv(DegCentImp4, "DegCentImpOneAndHalfKM4.csv")

#####
#####
#####
#part 5
NodeRemove5 <- list(NA)
DegreeCentralizationRemoval5 <- list(NA)
DegCentImp5 <- list(NA)
for(i in 1:1500){
	NodeRemove5[[i]] <- OneAndHalfKMNetwork
	NodeRemove5[[i]] <- NodeRemove5[[i]] - (6000 + i)
	DegreeCentralizationRemoval5[[i]] <- centralization.degree(NodeRemove5[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval5[[i]] <- DegreeCentralizationRemoval5[[i]]$centralization
	DegCentImp5[[i]] <- ((OneAndHalfKMNetworkDegreeCentrality - DegreeCentralizationRemoval5[[i]])/OneAndHalfKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp5 <- unlist(DegCentImp5)
DegCentImp5 <- data.frame(DegCentImp5)
DegCentImp5$centralization <- DegCentImp5[,1]
DegCentImp5$NodeID <- ContWetsAndPits@data[6001:7500,]$NodeID
DegCentImp5$FP_ID <- ContWetsAndPits@data[6001:7500,]$FP_ID
DegCentImp5 <- DegCentImp5[,-1]
head(DegCentImp5)
write.csv(DegCentImp5, "DegCentImpOneAndHalfKM5.csv")

#####
#####
#####
#part 6
NodeRemove6 <- list(NA)
DegreeCentralizationRemoval6 <- list(NA)
DegCentImp6 <- list(NA)
for(i in 1:1500){
	NodeRemove6[[i]] <- OneAndHalfKMNetwork
	NodeRemove6[[i]] <- NodeRemove6[[i]] - (7500 + i)
	DegreeCentralizationRemoval6[[i]] <- centralization.degree(NodeRemove6[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval6[[i]] <- DegreeCentralizationRemoval6[[i]]$centralization
	DegCentImp6[[i]] <- ((OneAndHalfKMNetworkDegreeCentrality - DegreeCentralizationRemoval6[[i]])/OneAndHalfKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp6 <- unlist(DegCentImp6)
DegCentImp6 <- data.frame(DegCentImp6)
DegCentImp6$centralization <- DegCentImp6[,1]
DegCentImp6$NodeID <- ContWetsAndPits@data[7501:9000,]$NodeID
DegCentImp6$FP_ID <- ContWetsAndPits@data[7501:9000,]$FP_ID
DegCentImp6 <- DegCentImp6[,-1]
head(DegCentImp6)
write.csv(DegCentImp6, "DegCentImpOneAndHalfKM6.csv")

#####
#####
#####
#part 7
NodeRemove7 <- list(NA)
DegreeCentralizationRemoval7 <- list(NA)
DegCentImp7 <- list(NA)
for(i in 1:910){
	NodeRemove7[[i]] <- OneAndHalfKMNetwork
	NodeRemove7[[i]] <- NodeRemove7[[i]] - (9000 + i)
	DegreeCentralizationRemoval7[[i]] <- centralization.degree(NodeRemove7[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval7[[i]] <- DegreeCentralizationRemoval7[[i]]$centralization
	DegCentImp7[[i]] <- ((OneAndHalfKMNetworkDegreeCentrality - DegreeCentralizationRemoval7[[i]])/OneAndHalfKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp7 <- unlist(DegCentImp7)
DegCentImp7 <- data.frame(DegCentImp7)
DegCentImp7$centralization <- DegCentImp7[,1]
DegCentImp7$NodeID <- ContWetsAndPits@data[9001:9910,]$NodeID
DegCentImp7$FP_ID <- ContWetsAndPits@data[9001:9910,]$FP_ID
DegCentImp7 <- DegCentImp7[,-1]
head(DegCentImp7)
write.csv(DegCentImp7, "DegCentImpOneAndHalfKM7.csv")

#read back in degree centrality importance scores
DegCentImp1 <- read.csv("DegCentImpOneAndHalfKM1.csv", header = TRUE)
DegCentImp1 <- DegCentImp1[,-1]
DegCentImp2 <- read.csv("DegCentImpOneAndHalfKM2.csv", header = TRUE)
DegCentImp2 <- DegCentImp2[,-1]
DegCentImp3 <- read.csv("DegCentImpOneAndHalfKM3.csv", header = TRUE)
DegCentImp3 <- DegCentImp3[,-1]
DegCentImp4 <- read.csv("DegCentImpOneAndHalfKM4.csv", header = TRUE)
DegCentImp4 <- DegCentImp4[,-1]
DegCentImp5 <- read.csv("DegCentImpOneAndHalfKM5.csv", header = TRUE)
DegCentImp5 <- DegCentImp5[,-1]
DegCentImp6 <- read.csv("DegCentImpOneAndHalfKM6.csv", header = TRUE)
DegCentImp6 <- DegCentImp6[,-1]
DegCentImp7 <- read.csv("DegCentImpOneAndHalfKM7.csv", header = TRUE)
DegCentImp7 <- DegCentImp7[,-1]

#combine csv files
DegCentImp <- rbind(DegCentImp1, DegCentImp2, DegCentImp3, DegCentImp4, DegCentImp5, DegCentImp6, DegCentImp7)

#add new outputs to attribute table
ContWetsAndPits@data$Degree <- DegreeOneAndHalfKMNetwork
ContWetsAndPits@data$DegCentImp <- DegCentImp$centralization
head(ContWetsAndPits@data)

#write updated spatial polygons data frame as an ESRI shapefile
writeOGR(ContWetsAndPits, "ContWetsAndPitsNonReservesOneAndHalfKM", "ContWetsAndPitsNonReservesOneAndHalfKM", driver = 'ESRI Shapefile', verbose = TRUE)

#####
#####
#####
#2.0 km
retain <- c("FromNode", "ToNode")
TwoKMPaths1 <- read.csv("TwoKMPathsNR1.csv")
TwoKMPaths1$FromNode <- TwoKMPaths1$unlist.FromNode1.
TwoKMPaths1$ToNode <- TwoKMPaths1$unlist.ToNode1.
TwoKMPaths1 <- TwoKMPaths1[retain]

TwoKMPaths2 <- read.csv("TwoKMPathsNR2.csv")
TwoKMPaths2$FromNode <- TwoKMPaths2$unlist.FromNode2.
TwoKMPaths2$ToNode <- TwoKMPaths2$unlist.ToNode2.
TwoKMPaths2 <- TwoKMPaths2[retain]

TwoKMPaths3 <- read.csv("TwoKMPathsNR3.csv")
TwoKMPaths3$FromNode <- TwoKMPaths3$unlist.FromNode3.
TwoKMPaths3$ToNode <- TwoKMPaths3$unlist.ToNode3.
TwoKMPaths3 <- TwoKMPaths3[retain]

TwoKMPaths4 <- read.csv("TwoKMPathsNR4.csv")
TwoKMPaths4$FromNode <- TwoKMPaths4$unlist.FromNode4.
TwoKMPaths4$ToNode <- TwoKMPaths4$unlist.ToNode4.
TwoKMPaths4 <- TwoKMPaths4[retain]

TwoKMPaths5 <- read.csv("TwoKMPathsNR5.csv")
TwoKMPaths5$FromNode <- TwoKMPaths5$unlist.FromNode5.
TwoKMPaths5$ToNode <- TwoKMPaths5$unlist.ToNode5.
TwoKMPaths5 <- TwoKMPaths5[retain]

TwoKMPaths6 <- read.csv("TwoKMPathsNR6.csv")
TwoKMPaths6$FromNode <- TwoKMPaths6$unlist.FromNode6.
TwoKMPaths6$ToNode <- TwoKMPaths6$unlist.ToNode6.
TwoKMPaths6 <- TwoKMPaths6[retain]

#combine 2.0 km connections into 1 data frame
TwoKMPathsCombined <- as.matrix(rbind(TwoKMPaths1, TwoKMPaths2, TwoKMPaths3, TwoKMPaths4, TwoKMPaths5, TwoKMPaths6))

#create network with nodes and associated edges
TwoKMNetwork <- graph(n = 9910, edges = TwoKMPathsCombined, directed = FALSE)

#calculate node and edge betweeness for graph (number of shortest paths going through node or edge)
TwoKMNodeBetweenness <- betweenness(TwoKMNetwork, v = V(TwoKMNetwork), directed = FALSE, weights = NULL, normalized = FALSE)
TwoKMNodeBetweenness
MeanNodeBetweennessTwoKM <- mean(TwoKMNodeBetweenness)
MeanNodeBetweennessTwoKM

TwoKMEdgeBetweenness <- edge.betweenness(TwoKMNetwork, e = E(TwoKMNetwork), directed = FALSE, weights = NULL)
TwoKMEdgeBetweenness
MeanEdgeBetweennessTwoKM <- mean(TwoKMEdgeBetweenness)
MeanEdgeBetweennessTwoKM

#calculate degree (number of adjacent edges for each node)
DegreeTwoKMNetwork <- degree(TwoKMNetwork, v = V(TwoKMNetwork), loops = TRUE)
DegreeTwoKMNetwork
MeanDegreeTwoKMNetwork <- mean(DegreeTwoKMNetwork)
MeanDegreeTwoKMNetwork

#find number of clusters (groups of connected graph components)
TwoKMNetworkNumberClusters <- no.clusters(TwoKMNetwork)
TwoKMNetworkNumberClusters

#find mean number of vertices per cluster
TwoKMNetworkClusters <- clusters(TwoKMNetwork)
summary(TwoKMNetworkClusters$csize)
TwoKMNetworkMeanClusterSize <- mean(TwoKMNetworkClusters$csize)
TwoKMNetworkMeanClusterSize

#calculate modularity
TwoKMNetworkModularity <- modularity(TwoKMNetwork, V(TwoKMNetwork))
TwoKMNetworkModularity

#find subcomponent (all other vertices reachable from specified vertice)
TwoKMNetworkSC <- list(NA)
NumberReachedTwoKM <- list(NA)
for(i in 1:nrow(ContWetsAndPits@data)){
	TwoKMNetworkSC[[i]] <- subcomponent(TwoKMNetwork, v = i)
	NumberReachedTwoKM[[i]] <- length(TwoKMNetworkSC[[i]]) - 1
}
NumberReachedTwoKM <- unlist(NumberReachedTwoKM)
NumberReachedTwoKMMean <- mean(NumberReachedTwoKM)
NumberReachedTwoKMMean

#edge betweeness community (creates hierarchical graph by removing edges most traveled when reaching other vertices via algorithm)
TwoKMNetworkEBC <- edge.betweenness.community(TwoKMNetwork, edge.betweenness = TRUE, merges = TRUE, bridges = TRUE)

#calculate eigenvector centrality scores of network positions (high scores = vertices connected to other highly connected which are connected to other highly connected etc...)
TwoKMNetworkEVCENT <- evcent(TwoKMNetwork, directed = FALSE, scale = FALSE)
TwoKMNetworkEVCENT

#calculate graph density (ratio of number of edges to number of possible edges)
TwoKMNetworkGraphDensity <- graph.density(TwoKMNetwork, loops = TRUE)
TwoKMNetworkGraphDensity

#identify neighbors of each vertice
TwoKMNetworkNeighbors <- list(NA)
for (i in 1:nrow(ContWetsAndPits@data)){
	TwoKMNetworkNeighbors[[i]] <- neighbors(TwoKMNetwork, v = i)
}
TwoKMNetworkNeighbors

#calculate neighborhood size (number of interconnected vertices in each neighborhood)
TwoKMNetworkNeighborSize <- neighborhood.size(TwoKMNetwork, order = 1, nodes = V(TwoKMNetwork))
TwoKMNetworkNeighborSize

#calculate graph cohesion (vertex connectivity of graph)
TwoKMNetworkGraphCohesion <- graph.cohesion(TwoKMNetwork)
TwoKMNetworkGraphCohesion

#calculate average shortest path length between connected vertices
TwoKMNetworkAPL <- average.path.length(TwoKMNetwork, directed = FALSE, unconnected = TRUE)
TwoKMNetworkAPL

#generate histogram of shortest path lengths
TwoKMNetworkAPLHist <- path.length.hist(TwoKMNetwork, directed = FALSE)
TwoKMNetworkAPLHist

#calculate degree and betweenness centrality for graph, without removing any nodes
TwoKMNetworkDegreeCentrality <- centralization.degree(TwoKMNetwork, loops = TRUE, normalized = TRUE)
TwoKMNetworkDegreeCentrality <- TwoKMNetworkDegreeCentrality$centralization
TwoKMNetworkDegreeCentrality

#add a degree and betweenness centralization importance scores for each vertice
#####
#####
#####
#part 1
NodeRemove1 <- list(NA)
DegreeCentralizationRemoval1 <- list(NA)
DegCentImp1 <- list(NA)
for(i in 1:1000){
	NodeRemove1[[i]] <- TwoKMNetwork
	NodeRemove1[[i]] <- NodeRemove1[[i]] - i
	DegreeCentralizationRemoval1[[i]] <- centralization.degree(NodeRemove1[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval1[[i]] <- DegreeCentralizationRemoval1[[i]]$centralization
	DegCentImp1[[i]] <- ((TwoKMNetworkDegreeCentrality - DegreeCentralizationRemoval1[[i]])/TwoKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp1 <- unlist(DegCentImp1)
DegCentImp1 <- data.frame(DegCentImp1)
DegCentImp1$centralization <- DegCentImp1[,1]
DegCentImp1$NodeID <- ContWetsAndPits@data[1:1000,]$NodeID
DegCentImp1$FP_ID <- ContWetsAndPits@data[1:1000,]$FP_ID
DegCentImp1 <- DegCentImp1[,-1]
head(DegCentImp1)
write.csv(DegCentImp1, "DegCentImpTwoKM1.csv")

#####
#####
#####
#part 2
NodeRemove2 <- list(NA)
DegreeCentralizationRemoval2 <- list(NA)
DegCentImp2 <- list(NA)
for(i in 1:1000){
	NodeRemove2[[i]] <- TwoKMNetwork
	NodeRemove2[[i]] <- NodeRemove2[[i]] - (1000 + i)
	DegreeCentralizationRemoval2[[i]] <- centralization.degree(NodeRemove2[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval2[[i]] <- DegreeCentralizationRemoval2[[i]]$centralization
	DegCentImp2[[i]] <- ((TwoKMNetworkDegreeCentrality - DegreeCentralizationRemoval2[[i]])/TwoKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp2 <- unlist(DegCentImp2)
DegCentImp2 <- data.frame(DegCentImp2)
DegCentImp2$centralization <- DegCentImp2[,1]
DegCentImp2$NodeID <- ContWetsAndPits@data[1001:2000,]$NodeID
DegCentImp2$FP_ID <- ContWetsAndPits@data[1001:2000,]$FP_ID
DegCentImp2 <- DegCentImp2[,-1]
head(DegCentImp2)
write.csv(DegCentImp2, "DegCentImpTwoKM2.csv")

#####
#####
#####
#part 3
NodeRemove3 <- list(NA)
DegreeCentralizationRemoval3 <- list(NA)
DegCentImp3 <- list(NA)
for(i in 1:1000){
	NodeRemove3[[i]] <- TwoKMNetwork
	NodeRemove3[[i]] <- NodeRemove3[[i]] - (2000 + i)
	DegreeCentralizationRemoval3[[i]] <- centralization.degree(NodeRemove3[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval3[[i]] <- DegreeCentralizationRemoval3[[i]]$centralization
	DegCentImp3[[i]] <- ((TwoKMNetworkDegreeCentrality - DegreeCentralizationRemoval3[[i]])/TwoKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp3 <- unlist(DegCentImp3)
DegCentImp3 <- data.frame(DegCentImp3)
DegCentImp3$centralization <- DegCentImp3[,1]
DegCentImp3$NodeID <- ContWetsAndPits@data[2001:3000,]$NodeID
DegCentImp3$FP_ID <- ContWetsAndPits@data[2001:3000,]$FP_ID
DegCentImp3 <- DegCentImp3[,-1]
head(DegCentImp3)
write.csv(DegCentImp3, "DegCentImpTwoKM3.csv")

#####
#####
#####
#part 4
NodeRemove4 <- list(NA)
DegreeCentralizationRemoval4 <- list(NA)
DegCentImp4 <- list(NA)
for(i in 1:1000){
	NodeRemove4[[i]] <- TwoKMNetwork
	NodeRemove4[[i]] <- NodeRemove4[[i]] - (3000 + i)
	DegreeCentralizationRemoval4[[i]] <- centralization.degree(NodeRemove4[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval4[[i]] <- DegreeCentralizationRemoval4[[i]]$centralization
	DegCentImp4[[i]] <- ((TwoKMNetworkDegreeCentrality - DegreeCentralizationRemoval4[[i]])/TwoKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp4 <- unlist(DegCentImp4)
DegCentImp4 <- data.frame(DegCentImp4)
DegCentImp4$centralization <- DegCentImp4[,1]
DegCentImp4$NodeID <- ContWetsAndPits@data[3001:4000,]$NodeID
DegCentImp4$FP_ID <- ContWetsAndPits@data[3001:4000,]$FP_ID
DegCentImp4 <- DegCentImp4[,-1]
head(DegCentImp4)
write.csv(DegCentImp4, "DegCentImpTwoKM4.csv")

#####
#####
#####
#part 5
NodeRemove5 <- list(NA)
DegreeCentralizationRemoval5 <- list(NA)
DegCentImp5 <- list(NA)
for(i in 1:1000){
	NodeRemove5[[i]] <- TwoKMNetwork
	NodeRemove5[[i]] <- NodeRemove5[[i]] - (4000 + i)
	DegreeCentralizationRemoval5[[i]] <- centralization.degree(NodeRemove5[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval5[[i]] <- DegreeCentralizationRemoval5[[i]]$centralization
	DegCentImp5[[i]] <- ((TwoKMNetworkDegreeCentrality - DegreeCentralizationRemoval5[[i]])/TwoKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp5 <- unlist(DegCentImp5)
DegCentImp5 <- data.frame(DegCentImp5)
DegCentImp5$centralization <- DegCentImp5[,1]
DegCentImp5$NodeID <- ContWetsAndPits@data[4001:5000,]$NodeID
DegCentImp5$FP_ID <- ContWetsAndPits@data[4001:5000,]$FP_ID
DegCentImp5 <- DegCentImp5[,-1]
head(DegCentImp5)
write.csv(DegCentImp5, "DegCentImpTwoKM5.csv")

#####
#####
#####
#part 6
NodeRemove6 <- list(NA)
DegreeCentralizationRemoval6 <- list(NA)
DegCentImp6 <- list(NA)
for(i in 1:1000){
	NodeRemove6[[i]] <- TwoKMNetwork
	NodeRemove6[[i]] <- NodeRemove6[[i]] - (5000 + i)
	DegreeCentralizationRemoval6[[i]] <- centralization.degree(NodeRemove6[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval6[[i]] <- DegreeCentralizationRemoval6[[i]]$centralization
	DegCentImp6[[i]] <- ((TwoKMNetworkDegreeCentrality - DegreeCentralizationRemoval6[[i]])/TwoKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp6 <- unlist(DegCentImp6)
DegCentImp6 <- data.frame(DegCentImp6)
DegCentImp6$centralization <- DegCentImp6[,1]
DegCentImp6$NodeID <- ContWetsAndPits@data[5001:6000,]$NodeID
DegCentImp6$FP_ID <- ContWetsAndPits@data[5001:6000,]$FP_ID
DegCentImp6 <- DegCentImp6[,-1]
head(DegCentImp6)
write.csv(DegCentImp6, "DegCentImpTwoKM6.csv")

#####
#####
#####
#part 7
NodeRemove7 <- list(NA)
DegreeCentralizationRemoval7 <- list(NA)
DegCentImp7 <- list(NA)
for(i in 1:1000){
	NodeRemove7[[i]] <- TwoKMNetwork
	NodeRemove7[[i]] <- NodeRemove7[[i]] - (6000 + i)
	DegreeCentralizationRemoval7[[i]] <- centralization.degree(NodeRemove7[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval7[[i]] <- DegreeCentralizationRemoval7[[i]]$centralization
	DegCentImp7[[i]] <- ((TwoKMNetworkDegreeCentrality - DegreeCentralizationRemoval7[[i]])/TwoKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp7 <- unlist(DegCentImp7)
DegCentImp7 <- data.frame(DegCentImp7)
DegCentImp7$centralization <- DegCentImp7[,1]
DegCentImp7$NodeID <- ContWetsAndPits@data[6001:7000,]$NodeID
DegCentImp7$FP_ID <- ContWetsAndPits@data[6001:7000,]$FP_ID
DegCentImp7 <- DegCentImp7[,-1]
head(DegCentImp7)
write.csv(DegCentImp7, "DegCentImpTwoKM7.csv")

#####
#####
#####
#part 8
NodeRemove8 <- list(NA)
DegreeCentralizationRemoval8 <- list(NA)
DegCentImp8 <- list(NA)
for(i in 1:1000){
	NodeRemove8[[i]] <- TwoKMNetwork
	NodeRemove8[[i]] <- NodeRemove8[[i]] - (7000 + i)
	DegreeCentralizationRemoval8[[i]] <- centralization.degree(NodeRemove8[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval8[[i]] <- DegreeCentralizationRemoval8[[i]]$centralization
	DegCentImp8[[i]] <- ((TwoKMNetworkDegreeCentrality - DegreeCentralizationRemoval8[[i]])/TwoKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp8 <- unlist(DegCentImp8)
DegCentImp8 <- data.frame(DegCentImp8)
DegCentImp8$centralization <- DegCentImp8[,1]
DegCentImp8$NodeID <- ContWetsAndPits@data[7001:8000,]$NodeID
DegCentImp8$FP_ID <- ContWetsAndPits@data[7001:8000,]$FP_ID
DegCentImp8 <- DegCentImp8[,-1]
head(DegCentImp8)
write.csv(DegCentImp8, "DegCentImpTwoKM8.csv")

#####
#####
#####
#part 9
NodeRemove9 <- list(NA)
DegreeCentralizationRemoval9 <- list(NA)
DegCentImp9 <- list(NA)
for(i in 1:1000){
	NodeRemove9[[i]] <- TwoKMNetwork
	NodeRemove9[[i]] <- NodeRemove9[[i]] - (8000 + i)
	DegreeCentralizationRemoval9[[i]] <- centralization.degree(NodeRemove9[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval9[[i]] <- DegreeCentralizationRemoval9[[i]]$centralization
	DegCentImp9[[i]] <- ((TwoKMNetworkDegreeCentrality - DegreeCentralizationRemoval9[[i]])/TwoKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp9 <- unlist(DegCentImp9)
DegCentImp9 <- data.frame(DegCentImp9)
DegCentImp9$centralization <- DegCentImp9[,1]
DegCentImp9$NodeID <- ContWetsAndPits@data[8001:9000,]$NodeID
DegCentImp9$FP_ID <- ContWetsAndPits@data[8001:9000,]$FP_ID
DegCentImp9 <- DegCentImp9[,-1]
head(DegCentImp9)
write.csv(DegCentImp9, "DegCentImpTwoKM9.csv")

#####
#####
#####
#part 10
NodeRemove10 <- list(NA)
DegreeCentralizationRemoval10 <- list(NA)
DegCentImp10 <- list(NA)
for(i in 1:910){
	NodeRemove10[[i]] <- TwoKMNetwork
	NodeRemove10[[i]] <- NodeRemove10[[i]] - (9000 + i)
	DegreeCentralizationRemoval10[[i]] <- centralization.degree(NodeRemove10[[i]], loops = TRUE, normalized = TRUE)
	DegreeCentralizationRemoval10[[i]] <- DegreeCentralizationRemoval10[[i]]$centralization
	DegCentImp10[[i]] <- ((TwoKMNetworkDegreeCentrality - DegreeCentralizationRemoval10[[i]])/TwoKMNetworkDegreeCentrality) * 100
}

#unlist degree importance scores for each node
DegCentImp10 <- unlist(DegCentImp10)
DegCentImp10 <- data.frame(DegCentImp10)
DegCentImp10$centralization <- DegCentImp10[,1]
DegCentImp10$NodeID <- ContWetsAndPits@data[9001:9910,]$NodeID
DegCentImp10$FP_ID <- ContWetsAndPits@data[9001:9910,]$FP_ID
DegCentImp10 <- DegCentImp10[,-1]
head(DegCentImp10)
write.csv(DegCentImp10, "DegCentImpTwoKM10.csv")

#read back in degree centrality importance scores
DegCentImp1 <- read.csv("DegCentImpTwoKM1.csv", header = TRUE)
DegCentImp1 <- DegCentImp1[,-1]
DegCentImp2 <- read.csv("DegCentImpTwoKM2.csv", header = TRUE)
DegCentImp2 <- DegCentImp2[,-1]
DegCentImp3 <- read.csv("DegCentImpTwoKM3.csv", header = TRUE)
DegCentImp3 <- DegCentImp3[,-1]
DegCentImp4 <- read.csv("DegCentImpTwoKM4.csv", header = TRUE)
DegCentImp4 <- DegCentImp4[,-1]
DegCentImp5 <- read.csv("DegCentImpTwoKM5.csv", header = TRUE)
DegCentImp5 <- DegCentImp5[,-1]
DegCentImp6 <- read.csv("DegCentImpTwoKM6.csv", header = TRUE)
DegCentImp6 <- DegCentImp6[,-1]
DegCentImp7 <- read.csv("DegCentImpTwoKM7.csv", header = TRUE)
DegCentImp7 <- DegCentImp7[,-1]
DegCentImp8 <- read.csv("DegCentImpTwoKM8.csv", header = TRUE)
DegCentImp8 <- DegCentImp8[,-1]
DegCentImp9 <- read.csv("DegCentImpTwoKM9.csv", header = TRUE)
DegCentImp9 <- DegCentImp9[,-1]
DegCentImp10 <- read.csv("DegCentImpTwoKM10.csv", header = TRUE)
DegCentImp10 <- DegCentImp10[,-1]

#combine csv files
DegCentImp <- rbind(DegCentImp1, DegCentImp2, DegCentImp3, DegCentImp4, DegCentImp5, DegCentImp6, DegCentImp7, DegCentImp8, DegCentImp9, DegCentImp10)

#add new outputs to attribute table
ContWetsAndPits@data$Degree <- DegreeTwoKMNetwork
ContWetsAndPits@data$DegCentImp <- DegCentImp$centralization
head(ContWetsAndPits@data)

#write updated spatial polygons data frame as an ESRI shapefile
writeOGR(ContWetsAndPits, "ContWetsAndPitsNonReservesTwoKM", "ContWetsAndPitsNonReservesTwoKM", driver = 'ESRI Shapefile', verbose = TRUE)
